@@ -75,12 +75,11 @@ from os.path import getmtime
 from .core import load_config
 from matplotlib.patches import Ellipse
 from scipy.signal import argrelextrema
-
-
+import warnings
 
 __all__ = [ 'a_rsun','f_m','m1sin3i','m2sin3i','asini','rhostar','g_2',
         'K_kms','m_comp','transit_width','esolve','t2z',
-        'tzero2tperi', 'vrad', 'xyz_planet']
+        'tperi2tzero','tzero2tperi', 'vrad', 'xyz_planet']
 
 _arsun   = (GM_SunN*mean_solar_day**2/(4*np.pi**2))**(1/3.)/R_SunN
 _f_m     = mean_solar_day*1e9/(2*np.pi)/GM_SunN
@@ -363,7 +362,12 @@ def t2z(t, tzero, P, sini, rstar, ecc=0, omdeg=90, returnMask=False):
         omrad = 0.5*np.pi
         z = np.sqrt(1 - np.cos(nu)**2*sini**2)/rstar
     else:
-        tp = tzero2tperi(tzero,P,sini,ecc,omdeg)
+        tp = tzero2tperi(tzero,P,sini,ecc,omdeg,return_nan_on_error=True)
+        if tp is np.nan:
+            if returnMask:
+                return np.full_like(t,np.nan),np.full_like(t,True,dtype=np.bool)
+            else:
+                return np.full_like(t,np.nan)
         M = 2*np.pi*(t-tp)/P
         E = esolve(M,ecc)
         nu = 2*np.arctan(np.sqrt((1+ecc)/(1-ecc))*np.tan(E/2))
@@ -378,7 +382,8 @@ def t2z(t, tzero, P, sini, rstar, ecc=0, omdeg=90, returnMask=False):
 
 #---------
 
-def tzero2tperi(tzero,P,sini,ecc,omdeg):
+def tzero2tperi(tzero,P,sini,ecc,omdeg,
+        return_nan_on_error=False):
     """
     Calculate time of periastron from time of mid-transit
 
@@ -397,10 +402,10 @@ def tzero2tperi(tzero,P,sini,ecc,omdeg):
      >>> tzero = 54321.6789
      >>> P = 1.23456
      >>> sini = 0.987
-     >>> ecc = 0.123
+     >>> ecc = 0.654
      >>> omdeg = 89.01
-     >>> print(tzero2tperi(tzero,P,sini,ecc,omdeg))
-     54321.6762764
+     >>> print("{:0.4f}".format(tzero2tperi(tzero,P,sini,ecc,omdeg)))
+     54321.6784
 
     """
     def _delta(th, sin2i, omrad, ecc):
@@ -429,19 +434,130 @@ def tzero2tperi(tzero,P,sini,ecc,omdeg):
                     t_ = t_[i_]
                 ta,tb,tc = (t_-0.01, t_, t_+0.01)
             except:
+                if return_nan_on_error: return np.nan
                 print(sin2i, omrad, ecc)
                 print(ta, tb, tc)
                 print(fa, fb, fc)
                 raise ValueError('tzero2tperi grid search fail')
-        theta = brent(_delta, args=(sin2i, omrad, ecc), brack=(ta, tb, tc))
+        try:
+            theta = brent(_delta, args=(sin2i, omrad, ecc), brack=(ta, tb, tc))
+        except ValueError:
+            if return_nan_on_error: return np.nan
+            print(sin2i, omrad, ecc)
+            print(ta, tb, tc)
+            print(fa, fb, fc)
+            raise ValueError('Not a bracketing interval.')
+
     if theta == np.pi:
         E = np.pi 
     else:
         E = 2*np.arctan(np.sqrt((1-ecc)/(1+ecc))*np.tan(theta/2))
     return tzero - (E - ecc*np.sin(E))*P/(2*np.pi)
 
+#---------
+
+def tperi2tzero(tperi,P,sini,ecc,omdeg,eclipse=False):
+    """
+    Calculate phase mid-eclipse from time of mid-transit
+
+    :param tperi: times of periastron passage
+    :param P: orbital period
+    :param sini: sine of orbital inclination 
+    :param ecc: eccentricity 
+    :param omdeg: longitude of periastron in degrees
+    :param eclipse: calculate time of mid-eclipse if True, else mid-transit
+
+    :returns: time of mid-eclipse 
+
+    :Example:
+     >>> from pycheops.funcs import tperi2tzero
+     >>> tperi = 54321.6784
+     >>> P = 1.23456
+     >>> sini = 0.987
+     >>> ecc = 0.654
+     >>> omdeg = 89.01
+     >>> t_transit = tperi2tzero(tperi,P,sini,ecc,omdeg)
+     >>> t_eclipse = tperi2tzero(tperi,P,sini,ecc,omdeg,eclipse=True)
+     >>> print(f"{t_transit:0.4f}, {t_eclipse:0.4f}")
+
+    """
+    def _delta(th, sin2i, omrad, ecc):
+        # Equation (4.9) from Hilditch
+        return (1-ecc**2)*(
+                np.sqrt(1-sin2i*np.sin(th+omrad)**2)/(1+ecc*np.cos(th)))
+
+    omrad = omdeg*np.pi/180
+    sin2i = sini**2
+    theta = 0.5*np.pi-omrad + np.pi*eclipse
+    if (1-sin2i) > np.finfo(0.).eps :
+        ta = theta-0.125*np.pi
+        tb = theta
+        tc = theta+0.125*np.pi
+        fa = _delta(ta, sin2i, omrad, ecc)
+        fb = _delta(tb, sin2i, omrad, ecc)
+        fc = _delta(tc, sin2i, omrad, ecc)
+        if ((fb>fa)|(fb>fc)):
+            t_ = np.linspace(0,2*np.pi,1024)
+            d_ = _delta(t_, sin2i, omrad, ecc)
+            try:
+                i_= argrelextrema(d_, np.less)[0]
+                t_ = t_[i_]
+                if len(t_)>1:
+                    i_ = (np.abs(t_ - tb)).argmin()
+                    t_ = t_[i_]
+                ta,tb,tc = (t_-0.01, t_, t_+0.01)
+            except:
+                print(sin2i, omrad, ecc)
+                print(ta, tb, tc)
+                print(fa, fb, fc)
+                raise ValueError('tzero2tperi grid search fail')
+        try:
+            theta = brent(_delta, args=(sin2i, omrad, ecc), brack=(ta, tb, tc))
+        except ValueError:
+            print(sin2i, omrad, ecc)
+            print(ta, tb, tc)
+            print(fa, fb, fc)
+            raise ValueError('Not a bracketing interval.')
+
+    if theta == np.pi:
+        E = np.pi 
+    else:
+        E = 2*np.arctan(np.sqrt((1-ecc)/(1+ecc))*np.tan(theta/2))
+    return tperi + (E - ecc*np.sin(E))*P/(2*np.pi)
+
 #---------------
 
+def eclipse_phase (P,sini,ecc,omdeg):
+    """
+    Calculate time of mid-transit/mid-eclipse from time of periastron
+
+    Uses the method by Lacy, 1992AJ....104.2213L
+
+    :param tzero: times of mid-transit
+    :param P: orbital period
+    :param sini: sine of orbital inclination 
+    :param ecc: eccentricity 
+    :param omdeg: longitude of periastron in degrees
+
+    :returns: phase of mid-eclipse
+
+    :Example:
+     >>> from pycheops.funcs import eclipse_phase
+     >>> P = 1.23456
+     >>> sini = 0.987
+     >>> ecc = 0.654
+     >>> omdeg = 89.01
+     >>> ph_ecl = eclipse_phase(tzero,P,sini,ecc,omdeg)
+     >>> print(f"Phase of eclipse = {ph_ecl:0.4f}")
+
+    """
+    t_peri = tzero2tperi(0,P,sini,ecc,omdeg)
+    t_ecl = tperi2tzero(t_peri,P,sini,ecc,omdeg,eclipse=True)
+    return t_ecl/P % 1
+
+#---------------
+
+
 def nu_max(Teff, logg):
     """
     Peak frequency in micro-Hz for solar-like oscillations.
@@ -549,7 +665,7 @@ def xyz_planet(t, tzero, P, sini, ecc=0, omdeg=90):
 
 def massradius(P=None, k=None, sini=None, ecc=None,
         m_star=None, r_star=None, K=None, aR=None,
-        jovian=False, verbose=True, return_samples=False,
+        jovian=False, solar=False, verbose=True, return_samples=False,
         plot=True, figsize=(8,6), xlim=None, ylim=None, 
         errorbar=True, err_kws={'capsize':4, 'color':'darkred', 'fmt':'o'},
         logmass=False, logradius=False, title=None,
@@ -613,6 +729,7 @@ def massradius(P=None, k=None, sini=None, ecc=None,
     mass, Earth radius and Earth density. Jovian mass, radius and density
     units can be selected by setting jovian=True. In both cases, the radius
     units are those for a sphere with the same volume as the Earth or Jupiter.
+    Alternatively, solar units can be selected using solar=True.
 
     The following statistics are calculated for each of the input and output
     quantities and are returned as a python dict.  
@@ -733,15 +850,22 @@ def massradius(P=None, k=None, sini=None, ecc=None,
                 ps[pn[_k]] = pv[_k][_j]
 
     # Generate samples for input parameters not already sampled
+    # N.B. All parameters assumed to be strictly positive so use abs() to
+    # avoid negative values.
     for n in set(pn) - set(ps.keys()):
         _i = pn.index(n)
-        ps[n] = None if pv[_i] is None else _s(pv[_i])
+        ps[n] = None if pv[_i] is None else np.abs(_s(pv[_i]))
 
     if jovian:
+        if solar: raise ValueError("Cannot specify both jovian and solar units")
         mfac = M_SunN/M_JupN 
         rfac = R_SunN/R_JupN 
         mstr = ' M_Jup'
         rstr = ' R_Jup'
+    elif solar:
+        mfac, rfac = 1, 1
+        mstr = ' M_Sun'
+        rstr = ' R_Sun'
     else:
         mfac = M_SunN/M_EarthN
         rfac = R_SunN/R_EarthN
@@ -809,6 +933,9 @@ def massradius(P=None, k=None, sini=None, ecc=None,
                 rho_Jup  = M_JupN / (4/3*np.pi*R_JupN**3)
                 ps['rho_p'] = _rho/rho_Jup
                 rhostr = ' rho_Jup'
+            elif solar:
+                ps['rho_p'] = _rho
+                rhostr = ' rho_Sun'
             else:
                 rho_Earth  = M_EarthN / (4/3*np.pi*R_EarthN**3)
                 ps['rho_p'] = _rho/rho_Earth
@@ -818,6 +945,10 @@ def massradius(P=None, k=None, sini=None, ecc=None,
                 print(parprint(_rho*1e-3,'rho_p',wn=8,w=10)+' [g.cm-3]')
             result['rho_p'] = _d(ps['rho_p'])
 
+    # Include input quantities in result
+    for k in ['P', 'k', 'sini', 'e', 'K', 'aR']:
+        result[k] = _d(ps[k])
+
     if return_samples:
         for k in result.keys():
             result[k]['sample'] = ps[k]
@@ -850,6 +981,9 @@ def massradius(P=None, k=None, sini=None, ecc=None,
     if jovian:
         ax.set_xlabel(r"$M/M_{\rm Jup}$", **lab_kws)
         ax.set_ylabel(r"$R/R_{\rm Jup}$", **lab_kws)
+    elif solar:
+        ax.set_xlabel(r"$M/M_{\odot}$", **lab_kws)
+        ax.set_ylabel(r"$R/R_{\odot}$", **lab_kws)
     else:
         ax.set_xlabel(r"$M/M_{\oplus}$", **lab_kws)
         ax.set_ylabel(r"$R/R_{\oplus}$", **lab_kws)
@@ -860,6 +994,8 @@ def massradius(P=None, k=None, sini=None, ecc=None,
     if zeng_models is not None:
         if jovian:
             mfac, rfac = M_EarthN/M_JupN, R_EarthN/R_JupN
+        elif solar:
+            mfac, rfac = M_EarthN/M_SunN, R_EarthN/R_SunN
         else:
             mfac, rfac = 1,1
         mfile = join(_model_path,'apj522803t2_mrt.txt')
@@ -872,6 +1008,8 @@ def massradius(P=None, k=None, sini=None, ecc=None,
     if baraffe_models is not None:
         if jovian:
             mfac, rfac = M_EarthN/M_JupN, 1
+        elif solar:
+            mfac, rfac = M_EarthN/M_SunN, R_JupN/R_SunN
         else:
             mfac, rfac = 1, R_JupN/R_EarthN
         mfile = join(_model_path,'aa9321-07_table4.csv')
@@ -895,10 +1033,14 @@ def massradius(P=None, k=None, sini=None, ecc=None,
             download = True
         if download:
             url = config['TEPCat']['download_url']
-            req=requests.post(url)
-            with open(TEPCatPath, 'wb') as file:
-                file.write(req.content)
-            if verbose:
+            try:
+                req=requests.post(url)
+            except:
+                warnings.warn("Failed to update TEPCat data file from server")
+            else:
+                with open(TEPCatPath, 'wb') as file:
+                    file.write(req.content)
+                if verbose:
                     print('TEPCat data downloaded from \n {}'.format(url))
         # Awkward table to deal with because of repeated column names
         T = Table.read(TEPCatPath,format='ascii.no_header')
@@ -909,6 +1051,9 @@ def massradius(P=None, k=None, sini=None, ecc=None,
         R_b = R_b[ok]
         if jovian:
             R_b = R_b*R_eJupN/R_JupN 
+        elif solar:
+            M_b = M_b*M_JupN/M_SunN
+            R_b = R_b*R_eJupN/R_SunN
         else:
             M_b = M_b*M_JupN/M_EarthN
             R_b = R_b*R_eJupN/R_EarthN 
