@@ -37,7 +37,6 @@ Functions
 
 from __future__ import (absolute_import, division, print_function,
                                 unicode_literals)
-
 import argparse
 import textwrap
 from astropy.table import Table, Row
@@ -93,7 +92,7 @@ SpTypeToTeff = {
 
 # Define a query object for Gaia DR2
 _query = """SELECT source_id, ra, dec, parallax, pmra, pmdec, \
-phot_g_mean_mag, bp_rp FROM gaiadr2.gaia_source \
+phot_g_mean_mag, phot_g_mean_flux_over_error, bp_rp FROM gaiadr2.gaia_source \
 WHERE CONTAINS(POINT('ICRS',gaiadr2.gaia_source.ra,gaiadr2.gaia_source.dec), \
  CIRCLE('ICRS',{},{},0.0666))=1 AND (phot_g_mean_mag<=16.5); \
 """
@@ -102,10 +101,6 @@ WHERE CONTAINS(POINT('ICRS',gaiadr2.gaia_source.ra,gaiadr2.gaia_source.dec), \
 # XML strings and formats for input for Feasibility checker and PHT2
 _xml_time_critical_fmt = """<?xml version="1.0" encoding="UTF-8"?>
 <!--                                                                          -->
-<!--         Template Observation Request file v. 9.1.1                       -->
-<!--                                                                          -->
-<!--         This file can be ingested in the CHEOPS Feasibility Checker      -->
-<!--                                                                          -->
 <!--         This file contains time-critical observations                    -->
 <!--                                                                          -->
 <!--         Generated by pycheops.make_xml_files                             -->
@@ -136,125 +131,43 @@ _xml_time_critical_fmt = """<?xml version="1.0" encoding="UTF-8"?>
     </Variable_Header>
   </Earth_Explorer_Header>
   
-  <!-- NOTE: The listed Request was selected from the GTO-P2 catalog  -->
   <Data_Block type="xml">
 
     <!-- TIME-CRITICAL REQUEST  -->
     <List_of_Time_Critical_Requests count="1">
       <Time_Critical_Request>
-        <!--  CHEOPSim : set the programme_ID  -->
         <Programme_ID>{:d}</Programme_ID>
-
-        <!--  CHEOPSim : set the observation_request_ID  -->
         <Observation_Request_ID>1</Observation_Request_ID>
-
-        <!--  PHT2, CHEOPSim : set the observation category ("time critical" or "non time critical")  -->
         <Observation_Category>time critical</Observation_Category>
-
-        <!--  PHT2, CHEOPSim : set the propietary period after first visit (maximum is 18 months = 547 days)  -->
         <Proprietary_Period_First_Visit unit="days">{:d}</Proprietary_Period_First_Visit>
-
-        <!--  PHT2, CHEOPSim : set the propietary period after last visit (maximum is 12 months = 365 days)  -->
         <Proprietary_Period_Last_Visit unit="days">{:d}</Proprietary_Period_Last_Visit>
-
-        <!--  PHT2, FC, CHEOPSim : set the target name  -->
         <Target_Name>{}</Target_Name>
-
-        <!--  PHT2, CHEOPSim/GTO : set the target GAIA_ID (https://gea.esac.esa.int/archive/)  -->
         <Gaia_ID>GAIA DR2 {}</Gaia_ID>
-
-        <!--  PHT2, CHEOPSim : set the target spectral type  -->
-        <Spectral_Type>{}V</Spectral_Type>
-
-        <!--  PHT2, FC, CHEOPSim : set the target V-band magnitude  -->
-        <Target_Vmagnitude unit="mag">{:0.2f}</Target_Vmagnitude>
-
-        <!--  PHT2, CHEOPSim : set the target V-band magnitude error  -->
-        <Target_Vmagnitude_Error unit="mag">{:0.2f}</Target_Vmagnitude_Error>
-
-        <!--  PHT2, CHEOPSim : set the readout mode  -->
+        <Spectral_Type>{}</Spectral_Type>
+        <Target_Magnitude unit="mag">{:0.2f}</Target_Magnitude>
+        <Target_Magnitude_Error unit="mag">{:0.2f}</Target_Magnitude_Error>
         <Readout_Mode>{}</Readout_Mode>
-
-        <!--  PHT2, FC, CHEOPSim : set the target R.A.  -->
         <Right_Ascension unit="deg">{:0.7f}</Right_Ascension>
-
-        <!--  PHT2, FC, CHEOPSim : set the target declination  -->
         <Declination unit="deg">{:0.7f}</Declination>
-
-        <!--  PHT2, CHEOPSim/GTO : set the target R.A. proper motion  -->
         <RA_Proper_Motion unit="mas/year">{:0.2f}</RA_Proper_Motion>
-
-        <!--  PHT2, CHEOPSim/GTO : set the target declination proper motion  -->
         <DEC_Proper_Motion unit="mas/year">{:0.2f}</DEC_Proper_Motion>
-
-        <!--  PHT2, CHEOPSim/GTO : set the target parallax  -->
         <Parallax unit="mas">{:0.2f}</Parallax>
-
-        <!--  PHT2, CHEOPSim/GTO : set the target Effective temperature  -->
         <T_Eff unit="Kelvin">{:0.0f}</T_Eff>
-
-        <!--  PHT2 : set the V-band extinction toward the target  -->
         <Extinction unit="mag">0.00</Extinction>
-
-        <!--  The following 2 parameters are used to bound the period when the visits should be scheduled  -->
-        <!--  e.g. 15 Jan 2019 = 2458498.5 (JD)  -->
-        <!--  e.g. 15 Mar 2019 = 2458529.5 (JD)  -->
-        <!--  These 2 parameters are optional and can be commented out if not relevant for this observation  -->
-        <!--  PHT2, FC : set the earliest start date (in BJD) for this observation request  -->
         <Earliest_Start unit="BJD">{:0.3f}</Earliest_Start>
-
-        <!--  PHT2, FC : set the latest start date (in BJD) for this observation request  -->
         <Latest_End unit="BJD">{:0.3f}</Latest_End>
-
-        <!--  PHT2, CHEOPSim : set the exposure time for this observation request   -->
         <Exposure_Time unit="sec">{:0.2f}</Exposure_Time>
-
-        <!--  PHT2, CHEOPSim : set the exposure repetition period (optional) . Use fastest repetition period if not specified -->
         <Number_Stacked_Images>{:d}</Number_Stacked_Images>
-        
-        <!-- PHT2, CHEOPSim : mandatory but not relevant for the FC  -->
         <Number_Stacked_Imagettes>{:d}</Number_Stacked_Imagettes>
-
-        <!--  PHT2, FC, CHEOPSim/GTO : set the central time of the transit (in BJD) (time critical only)  -->
         <Transit_Time unit="BJD">{:0.6f}</Transit_Time>
-
-        <!--  PHT2, FC, CHEOPSim/GTO : set the orbital period of the planet (in days) (time critical only)  -->
         <Transit_Period unit="days">{:0.6f}</Transit_Period>
-
-        <!--  1 orbit  =  6000 seconds  -->
-        <!--  PHT2, FC, CHEOPSim : set the visit duration (in seconds)  -->
-        <Visit_Duration unit="sec">{:d}</Visit_Duration>
-
-        <!--  PHT2 : set the number of requested visit for this observation request  -->
+        <Visit_Duration unit="sec">{:0.1f}</Visit_Duration>
         <Number_of_Visits>{:d}</Number_of_Visits>
-
         <Continuous_Visits>false</Continuous_Visits>   <!--  Irrelevant for nominal science observations  -->
-
-        <!--  PHT2 : set the priority of the observation request (1 = A-grade, 2 = B-grade, 3 = C-grade)  -->
         <Priority>{:d}</Priority>
-
-        <!-- This parameter defines the minimum on-source time relative to the visit duration  -->
-        <!-- (excluding interruptions due to the SAA and Earth Occultations)  -->
-        <!--  PHT2, FC : set the minimum effective duration, ~ observing efficiency (in percent)  -->
         <Minimum_Effective_Duration unit="%">{:d}</Minimum_Effective_Duration>
-
-        <!--  This parameter defines the flexibility of the visit start time in units of planetary orbital phase.  -->
-        <!--  Two values are given to define the allowed start time of the visit.  -->
-        <!--  You should set "Earliest" strictly lower than "Latest" and the equivalent time-difference between  -->
-        <!--    "Earliest" and "Latest" should be longer than 30 minutes  (limitation of current version of the tool)  -->
-        <!--  PHT2, FC : set the start phase between 0 and 1 (mid-transit occurs at phase = 0)  -->
         <Earliest_Observation_Start unit="phase">{:0.4f}</Earliest_Observation_Start>
         <Latest_Observation_Start unit="phase">{:0.4f}</Latest_Observation_Start>
-
-    	<!--  Optional  -->
-    	<!--  The set of parameters below is used to define specific (orbital) phase ranges  -->
-    	<!--  within which the observing efficiency may be increased to a specific value  -->
-    	<!--  Convention is that the transit is at phase=0 (or equivalently 1)  -->
-    	<!--  This can be seen as a local requirement on the observing efficiency (e.g. egresses)  -->
-    	<!--  In this version, you should input "Start" < "End" (will be improved in future version)  -->
-    	<!--  PHT2, FC : set the critical phase ranges  -->
-{}
-
         <Send_Data_Taking_During_SAA>false</Send_Data_Taking_During_SAA>
         <Send_Data_Taking_During_Earth_Constraints>false</Send_Data_Taking_During_Earth_Constraints>
         <PITL>true</PITL>
@@ -267,11 +180,6 @@ _xml_time_critical_fmt = """<?xml version="1.0" encoding="UTF-8"?>
 
 _xml_non_time_critical_fmt = """<?xml version="1.0" encoding="UTF-8"?>
 <!--                                                               -->
-<!--  Template Observation Request file v. 9.1.1                   -->
-<!--                                                               -->
-<!--  This file can be ingested in the CHEOPS Feasibility Checker  -->
-<!--                                                               -->
-<!--                                                               -->
 <!--  This file contains non-time-critical observations            -->
 <!--                                                               -->
 <!--  Generated by pycheops.make_xml_files                         -->
@@ -302,103 +210,38 @@ _xml_non_time_critical_fmt = """<?xml version="1.0" encoding="UTF-8"?>
     </Variable_Header>
   </Earth_Explorer_Header>
   
-  <!-- NOTE: The listed Request was selected from the GTO-P2 catalog  -->
   <Data_Block type="xml">
-
     <!-- NON-TIME-CRITICAL REQUEST  -->
     <List_of_Non_Time_Critical_Requests count="1">
       <Non_Time_Critical_Request>
-        <!--  CHEOPSim : set the programme_ID  -->
         <Programme_ID>{:d}</Programme_ID>
-    
-        <!--  CHEOPSim : set the observation_request_ID  -->
         <Observation_Request_ID>1</Observation_Request_ID>
-    
-        <!--  PHT2, CHEOPSim : set the observation category ("time critical" or "non time critical")  -->
         <Observation_Category>non time critical</Observation_Category>
-    
-        <!--  PHT2, CHEOPSim : set the propietary period after first visit (maximum is 18 months = 547 days)  -->
         <Proprietary_Period_First_Visit unit="days">{:d}</Proprietary_Period_First_Visit>
-    
-        <!--  PHT2, CHEOPSim : set the propietary period after last visit (maximum is 12 months = 365 days)  -->
         <Proprietary_Period_Last_Visit unit="days">{:d}</Proprietary_Period_Last_Visit>
-    
-        <!--  PHT2, FC, CHEOPSim : set the target name  -->
         <Target_Name>{}</Target_Name>
-    
-        <!--  PHT2, CHEOPSim/GTO : set the target GAIA_ID (https://gea.esac.esa.int/archive/)  -->
         <Gaia_ID>GAIA DR2 {}</Gaia_ID>
-    
-        <!--  PHT2, CHEOPSim : set the target spectral type  -->
-        <Spectral_Type>{}V</Spectral_Type>
-    
-        <!--  PHT2, FC, CHEOPSim : set the target V-band magnitude  -->
-        <Target_Vmagnitude unit="mag">{:0.2f}</Target_Vmagnitude>
-    
-        <!--  PHT2, CHEOPSim : set the target V-band magnitude error  -->
-        <Target_Vmagnitude_Error unit="mag">{:0.2f}</Target_Vmagnitude_Error>
-    
-        <!--  PHT2, CHEOPSim : set the readout mode  -->
+        <Spectral_Type>{}</Spectral_Type>
+        <Target_Magnitude unit="mag">{:0.2f}</Target_Magnitude>
+        <Target_Magnitude_Error unit="mag">{:0.2f}</Target_Magnitude_Error>
         <Readout_Mode>{}</Readout_Mode>
-    
-        <!--  PHT2, FC, CHEOPSim : set the target R.A.  -->
         <Right_Ascension unit="deg">{:0.5f}</Right_Ascension>
-    
-        <!--  PHT2, FC, CHEOPSim : set the target declination  -->
         <Declination unit="deg">{:0.5f}</Declination>
-    
-        <!--  PHT2, CHEOPSim/GTO : set the target R.A. proper motion  -->
         <RA_Proper_Motion unit="mas/year">{:0.2f}</RA_Proper_Motion>
-    
-        <!--  PHT2, CHEOPSim/GTO : set the target declination proper motion  -->
         <DEC_Proper_Motion unit="mas/year">{:0.2f}</DEC_Proper_Motion>
-    
-        <!--  PHT2, CHEOPSim/GTO : set the target parallax  -->
         <Parallax unit="mas">{:0.2f}</Parallax>
-    
-        <!--  PHT2, CHEOPSim/GTO : set the target Effective temperature  -->
         <T_Eff unit="Kelvin">{:0.0f}</T_Eff>
-    
-        <!--  PHT2 : set the V-band extinction toward the target  -->
         <Extinction unit="mag">0.00</Extinction>
-    
-        <!--  The following 2 parameters are used to bound the period when the visits should be scheduled  -->
-        <!--  e.g. 15 Jan 2019 = 2458498.5 (JD)  -->
-        <!--  e.g. 15 Mar 2019 = 2458529.5 (JD)  -->
-        <!--  These 2 parameters are optional and can be commented out if not relevant for this observation  -->
-        <!--  PHT2, FC : set the earliest start date (in BJD) for this observation request  -->
         <Earliest_Start unit="BJD">{:0.3f}</Earliest_Start>
-    
-        <!--  PHT2, FC : set the latest start date (in BJD) for this observation request  -->
         <Latest_End unit="BJD">{:0.3f}</Latest_End>
-    
-        <!--  PHT2, CHEOPSim : set the exposure time for this observation request   -->
         <Exposure_Time unit="sec">{:0.2f}</Exposure_Time>
-    
-        <!--  PHT2, CHEOPSim : set the exposure repetition period (optional) . Use fastest repetition period if not specified -->
         <Number_Stacked_Images>{:d}</Number_Stacked_Images>
-        
-        <!-- PHT2, CHEOPSim : mandatory but not relevant for the FC  -->
         <Number_Stacked_Imagettes>{:d}</Number_Stacked_Imagettes>
-
-        <!--  1 orbit  =  6000 seconds  -->
-        <!--  PHT2, FC, CHEOPSim : set the visit duration (in seconds)  -->
-        <Visit_Duration unit="sec">{:d}</Visit_Duration>
-    
-        <!--  PHT2 : set the number of requested visit for this observation request  -->
+        <Visit_Duration unit="sec">{:0.1f}</Visit_Duration>
         <Number_of_Visits>{:d}</Number_of_Visits>
-    
         <Continuous_Visits>false</Continuous_Visits>   <!--  Irrelevant for nominal science observations  -->
-    
-        <!--  PHT2 : set the priority of the observation request (1 = A-grade, 2 = B-grade, 3 = C-grade)  -->
         <Priority>{:d}</Priority>
-    
-    
-        <!-- This parameter defines the minimum on-source time relative to the visit duration  -->
-        <!-- (excluding interruptions due to the SAA and Earth Occultations)  -->
-        <!--  PHT2, FC : set the minimum effective duration, ~ observing efficiency (in percent)  -->
         <Minimum_Effective_Duration unit="%">{:d}</Minimum_Effective_Duration>
-    
         <Send_Data_Taking_During_SAA>false</Send_Data_Taking_During_SAA>
         <Send_Data_Taking_During_Earth_Constraints>false</Send_Data_Taking_During_Earth_Constraints>
         <PITL>true</PITL>
@@ -464,6 +307,7 @@ def _GaiaDR2Match(row, fC, match_radius=1,  gaia_mag_tolerance=0.5,
     DR2Table['pmra'].unit = 'mas / yr'
     DR2Table['pmdec'].unit = 'mas / yr'
 
+
     cat = SkyCoord(DR2Table['ra'],DR2Table['dec'],
             frame='icrs',
             distance=Distance(parallax=DR2Table['parallax'].quantity),
@@ -479,11 +323,19 @@ def _GaiaDR2Match(row, fC, match_radius=1,  gaia_mag_tolerance=0.5,
         GV = SpTypeToGminusV[key]
     except TypeError:
         flags += 1024
-        GV = 0.0
-    Gmag = row['Vmag'] + GV
+        GV = -0.15
+    try:
+        Gmag = float(row['Gmag'])
+    except ValueError:
+        raise ValueError('Invalid Gmag value ',row['Gmag'])
+    except KeyError:
+        Gmag = row['Vmag'] + GV
 
     if abs(Gmag-DR2Table['phot_g_mean_mag'][idx]) > gaia_mag_tolerance:
-        print("Input values: V = {:5.2f}, SpTy = {} -> G_est = {:5.2f}"
+        if 'Gmag' in row.colnames:
+            print("Input value: G = ", Gmag)
+        else:
+            print("Input values: V = {:5.2f}, SpTy = {} -> G_est = {:5.2f}"
                 .format(row['Vmag'], row['SpTy'], Gmag))
         print("Catalogue values: G = {:5.2f}, Source = {}"
                 .format(DR2Table['phot_g_mean_mag'][idx], 
@@ -686,8 +538,9 @@ def _parcheck_time_critical(Priority, MinEffDur,
     return _parcheck_non_time_critical(Priority, MinEffDur, Earliest_start_date,
         Latest_end_date)
 
-def _target_table_row_to_xml(row, 
-        progamme_id=0, proprietary_first=547, proprietary_last=365):
+def _target_table_row_to_xml(row, progamme_id=0,
+        proprietary_first=547, proprietary_last=365, 
+        checker=False, user_g_mag=False):
 
     period = row['Period'] 
     t_exp = row['T_exp']
@@ -700,7 +553,23 @@ def _target_table_row_to_xml(row,
             pm_dec=row['pmdec']*u.mas/u.yr )
     radeg = float(c.to_string(precision=5).split()[0])
     dedeg = float(c.to_string(precision=5).split()[1])
-    
+
+    if user_g_mag:
+        gmag = row['Gmag']
+        e_gmag = row['e_Gmag']
+    else:
+        gmag = row['dr2_g_mag']
+        e_gmag = row['e_dr2_g_mag']
+
+    if checker:
+        try:
+            key = re.match('[AFGKM][0-9]', row['SpTy'])[0]
+            GV = SpTypeToGminusV[key]
+        except TypeError:
+            GV = -0.15
+        mag, e_mag = gmag-GV, e_gmag
+    else:
+        mag, e_mag = gmag, e_gmag
 
     if period > 0:
         error = _parcheck_time_critical(
@@ -719,7 +588,7 @@ def _target_table_row_to_xml(row,
               _creation_time_string(),
               progamme_id, proprietary_first, proprietary_last,
               row['Target'], row['Gaia_DR2'], row['SpTy'],
-              row['Vmag'], row['e_Vmag'], 
+              mag, e_mag,
               _choose_romode(t_exp),
               radeg, dedeg, row['pmra'], row['pmdec'],
               row['parallax'], row['T_eff'], 
@@ -747,7 +616,7 @@ def _target_table_row_to_xml(row,
               _creation_time_string(),
               progamme_id, proprietary_first, proprietary_last,
               row['Target'], row['Gaia_DR2'], row['SpTy'],
-              row['Vmag'], row['e_Vmag'], 
+              mag, e_mag,
               _choose_romode(t_exp),
               radeg, dedeg, row['pmra'], row['pmdec'],
               row['parallax'], row['T_eff'],
@@ -756,6 +625,10 @@ def _target_table_row_to_xml(row,
               row['T_visit'], row['N_Visits'], row['Priority'],
               row['MinEffDur']
               )
+
+    if checker:
+        xml = xml.replace('Target_Magnitude','Target_Vmagnitude')
+
     return xml
 
 def main():
@@ -772,17 +645,23 @@ def main():
         The target for each observation is defined by the input _RAJ2000 and
         _DEJ2000 coordinates. There must be a matching source in Gaia DR2 for
         each input coordinate. The G-band magnitude of the source must also
-        match the G-band magnitude estimated from Vmag and SpTy. The following
-        table is an abbreviated version of the look-up table used to estimate
-        the G-band magnitude from  Vmag and SpTy.
+        match the G-band magnitude provided in the input table, or estimated
+        from Vmag and SpTy from the same table if Gmag is not given. The
+        following table is an abbreviated version of the look-up table used to
+        estimate the G-band magnitude from  Vmag and SpTy.
 
-          SpTy G-V             SpTy G-V
-          -----------          -----------
-          A0   -0.019          K0   -0.229
-          F0   -0.078          K5   -0.453
-          F5   -0.109          M0   -0.997
-          G0   -0.153          M5   -2.979
-          G5   -0.181          M9   -3.337
+          SpTy G-V    Teff/K
+          ------------------
+          A0   -0.019   9700
+          F5   -0.109   6510
+          G5   -0.181   5660
+          K0   -0.229   5280
+          M0   -0.997   3870
+          M9   -3.337   2400
+
+        N.B. An estimate of the spectral type is needed in any case for
+        observation requests because accurate flat-fielding requires an
+        estimate of the star's effective temperature.
 
         The input table can be any format suitable for reading with the
         command astropy.table.Table.read(), e.g., CSV.
@@ -794,8 +673,6 @@ def main():
          _RAJ2000   - right ascension, ICRS epoch J2000.0, hh:mm:ss.ss
          _DEJ2000   - declination, ICRS epoch J2000.0, +dd:mm:ss.s
          SpTy       - spectral type (any string starting [AFGKM][0-9])
-         Vmag       - V-band magnitude
-         e_Vmag     - error in V-band magnitude
          BJD_early  - earliest start date (BJD)
          BJD_late   - latest start date (BJD) 
          T_visit    - visit duration in seconds
@@ -803,6 +680,13 @@ def main():
          Priority   - 1, 2 or 3 
          MinEffDur  - minimum on-source time, percentage of T_visit (integer)
 
+         In addition, the input table must specify either ...
+         Gmag       - G-band magnitude
+         e_Gmag     - error in g-band magnitude
+         ... or ...
+         Vmag       - V-band magnitude
+         e_Vmag     - error in V-band magnitude
+
         If the flag --ignore-gaia-id-check is not specified on the command
         line then the following column is also required.
          Gaia_DR2   - Gaia DR2 identification number (integer)
@@ -870,7 +754,7 @@ def main():
              +  8192 = Acquisition warning, brighter star within 51"-180"
              +  4096 = Contamination error, Contam > 1
              +  2048 = Contamination warning, Contam = 0.1 - 1
-             +  1024 = No spectral type match, assuming G-V = 0 
+             +  1024 = No spectral type match, assuming G-V = -0.15
              +  512 = Visibility error, efficiency = 0
              +  256 = Visibility warning, efficiency < 50%
              +  128 = Exposure time error - target will be saturated
@@ -917,6 +801,17 @@ def main():
         '''
     )
 
+    parser.add_argument('-u', '--use_gaia_mag_from_table', 
+        action='store_const',
+        dest='user_g_mag',
+        const=True,
+        default=False,
+        help='''
+        Use Gaia magnitude from the input table instead of Gaia DR2 value for
+        calculation and in the output XML file.
+        '''
+    )
+
     parser.add_argument('--ignore-gaia-id-check',
         action='store_const',
         dest='id_check',
@@ -957,14 +852,30 @@ def main():
         help='Overwrite existing output files.'
     )
 
+    parser.add_argument('-c', '--checker', 
+        action='store_const',
+        dest='checker',
+        const=True,
+        default=False,
+        help='Output XML suitable for use with Scheduling Feasibility Checker'
+    )
+
     parser.add_argument('-x', '--suffix', 
-        default='.xml', type=str,
+        default='_EXT_APP_ObservationRequests.xml', type=str,
         help='''
         Output file name suffix
         (default: %(default)s)
         '''
     )
 
+    parser.add_argument('-d', '--directory', 
+        default='.', type=str,
+        help='''
+        Output directory for xml files 
+        (default: %(default)s)
+        '''
+    )
+
     parser.add_argument('--proprietary_last', 
         default=365, type=int,
         help='Propietary period after last visit'
@@ -1031,7 +942,7 @@ def main():
         except KeyError:
             table[key] = -1
 
-    for key in ('pmra', 'pmdec', 'parallax'):
+    for key in ('pmra', 'pmdec', 'parallax', 'dr2_g_mag', 'e_dr2_g_mag'):
         try:
             table['Old_{}'.format(key)] = table[key]
         except KeyError:
@@ -1046,6 +957,10 @@ def main():
     if not 'N_Ranges' in table.columns:
         table['N_Ranges'] = 0
 
+    # Ensure RA and Dec columns are wide enough to accept updates RA/Dec
+    # values from Gaia catalogue to full precision
+    table['_RAJ2000'] =  [s.rjust(11) for s in table['_RAJ2000']]
+    table['_DEJ2000'] =  [s.rjust(11) for s in table['_DEJ2000']]
 
     # Load contamination function from pickle
     config = load_config()
@@ -1089,14 +1004,17 @@ def main():
 
     print('#{}'.format(ObsReqNameHeader) + tstr.format(ObsReqNameHeader))
 
-    # String of coordinates, Vmag and SpTy to enable re-use of DR2 data
+    # String of coordinates, Vmag/Gmag and SpTy to enable re-use of DR2 data
     old_tag = None
     for row in table:
 
         coo = SkyCoord(row['_RAJ2000'],row['_DEJ2000'],
               frame='icrs',unit=(u.hourangle, u.deg))
 
-        tag = "{}, {}, {}".format(coo.to_string(), row['Vmag'], row['SpTy'])
+        if 'Gmag' in row.colnames:
+            tag = "{}, {}, {}".format(coo.to_string(), row['Gmag'], row['SpTy'])
+        else:
+            tag = "{}, {}, {}".format(coo.to_string(), row['Vmag'], row['SpTy'])
         if tag != old_tag:
             old_tag = tag
             DR2data,contam,flags,coords = _GaiaDR2Match(row, fC, rtol, gtol,
@@ -1111,6 +1029,8 @@ def main():
         row['pmra'] = DR2data['pmra']
         row['pmdec'] = DR2data['pmdec']
         row['parallax'] = DR2data['parallax']
+        row['dr2_g_mag'] = DR2data['phot_g_mean_mag']
+        row['e_dr2_g_mag'] = 1.086/DR2data['phot_g_mean_flux_over_error']
 
         try:
             if row['Old_T_eff'] <= 0:
@@ -1125,7 +1045,10 @@ def main():
                 row['T_eff'] = 5999
 
         _T = row['T_eff']
-        _G = DR2data['phot_g_mean_mag']
+        if args.user_g_mag:
+            _G = row['Gmag']
+        else:
+            _G = DR2data['phot_g_mean_mag']
         if args.auto_expose:
             row['T_exp'] = exposure_time(_G, _T, 
                     frac=args.scaling_factor_percent/100)
@@ -1139,14 +1062,19 @@ def main():
             flags += 64
 
         xmlfile = "{}{}".format(row['ObsReqName'],args.suffix)
-        if exists(xmlfile) and not args.overwrite:
+        if args.directory is None:
+            xmlpath = xmlfile
+        else:
+            xmlpath = join(args.directory, xmlfile)
+        if exists(xmlpath) and not args.overwrite:
             raise IOError("Output file {} exists, use -f option to overwrite"
-                    .format(xmlfile))
-        f = open(xmlfile,'w')
+                    .format(xmlpath))
+        f = open(xmlpath,'w')
         f.write(_target_table_row_to_xml(row,
                     progamme_id=args.programme_id, 
                     proprietary_first=args.proprietary_first,
-                    proprietary_last=args.proprietary_last)
+                    proprietary_last=args.proprietary_last,
+                    checker=args.checker, user_g_mag=args.user_g_mag)
             )
         f.close()
 
@@ -1158,6 +1086,7 @@ def main():
 
         c_tot, c_av, c_max = count_rate(_G, row['T_exp'])
         print(TerminalOutputFormat.format( row['ObsReqName'],
-            DR2data['source_id'], _G, coords.ra.degree, coords.dec.degree,
+            DR2data['source_id'], DR2data['phot_g_mean_mag'],
+            coords.ra.degree, coords.dec.degree,
             contam, vis, row['T_exp'],flags, c_tot, 100*frac, duty, img, igt))
 
