@@ -348,7 +348,8 @@ class TransitModel(Model):
             c2 = 1 - h_1 + h_2
             a2 = np.log2(c2/h_2)
             z,m = t2z(t, T_0, P, sini, r_star, ecc, om, returnMask = True)
-            # Set z values where planet is behind star to a large nominal value
+            if False in np.isfinite(z): return np.ones_like(t)
+            # Set z values where planet is behind star to a big nominal value
             z[m]  = 100
             return qpower2(z, k, c2, a2)
 
@@ -356,21 +357,28 @@ class TransitModel(Model):
         self._set_paramhints_prefix()
 
     def _set_paramhints_prefix(self):
-        self.set_param_hint('P', min=1e-15)
-        self.set_param_hint('D', min=0, max=0.25)
-        self.set_param_hint('W', min=0, max=0.3)
-        self.set_param_hint('b', min=0, max=1.0)
-        self.set_param_hint('f_c', value=0, min=-1, max=1, vary=False)
-        self.set_param_hint('f_s', value=0, min=-1, max=1, vary=False)
-        self.set_param_hint('h_1', value=0.7224, min=0, max=1, vary=False)
-        self.set_param_hint('h_2', value=0.6713, min=0, max=1, vary=False)
+        p = self.prefix
+        self.set_param_hint(f'{p}P', value=1, min=1e-15)
+        self.set_param_hint(f'{p}D', value=0.01, min=0, max=0.25)
+        self.set_param_hint(f'{p}W', value=0.1, min=0, max=0.3)
+        self.set_param_hint(f'{p}b', value=0.3, min=0, max=1.0)
+        self.set_param_hint(f'{p}f_c', value=0, min=-1, max=1, vary=False)
+        self.set_param_hint(f'{p}f_s', value=0, min=-1, max=1, vary=False)
+        expr = "{p:s}f_c**2 + {p:s}f_s**2".format(p=self.prefix)
+        self.set_param_hint(f'{p}e',min=0,max=1,expr=expr)
+        self.set_param_hint(f'{p}h_1', value=0.7224,min=0,max=1,vary=False)
+        self.set_param_hint(f'{p}h_2', value=0.6713,min=0,max=1,vary=False)
+        expr = "(1-{p:s}h_2)**2".format(p=self.prefix)
+        self.set_param_hint(f'{p}q_1',min=0,max=1,expr=expr)
+        expr = "({p:s}h_1-{p:s}h_2)/(1-{p:s}h_2)".format(p=self.prefix)
+        self.set_param_hint(f'{p}q_2',min=0,max=1,expr=expr)
         expr = "sqrt({p:s}D)".format(p=self.prefix)
-        self.set_param_hint('k'.format(p=self.prefix), 
+        self.set_param_hint(f'{p}k'.format(p=self.prefix), 
                 expr=expr, min=0, max=0.5)
         expr ="sqrt((1+{p:s}k)**2-{p:s}b**2)/{p:s}W/pi".format(p=self.prefix)
-        self.set_param_hint('aR',min=1, expr=expr)
+        self.set_param_hint(f'{p}aR',min=1, expr=expr)
         expr = "0.013418*{p:s}aR**3/{p:s}P**2".format(p=self.prefix)
-        self.set_param_hint('rho', min=0, expr = expr)
+        self.set_param_hint(f'{p}rho', min=0, expr = expr)
 
 
 #----------------------
@@ -437,6 +445,7 @@ class EclipseModel(Model):
             if ecc > 0.95 : return np.ones_like(t)
             om = np.arctan2(f_s, f_c)*180/np.pi
             z,m = t2z(t-a_c, T_0, P, sini, r_star, ecc, om, returnMask=True)
+            if False in np.isfinite(z): return np.ones_like(t)
             # Set z values where star is behind planet to a large nominal value
             z[~m]  = 100
             return 1 + L*(ueclipse(z, k)-1)
@@ -445,57 +454,63 @@ class EclipseModel(Model):
         self._set_paramhints_prefix()
 
     def _set_paramhints_prefix(self):
-        self.set_param_hint('P', min=1e-15)
-        self.set_param_hint('D', min=0, max=0.25)
-        self.set_param_hint('W', min=0, max=0.3)
-        self.set_param_hint('b', min=0, max=1.0)
-        self.set_param_hint('L', min=0, max=1)
-        self.set_param_hint('f_c', value=0, min=-1, max=1, vary=False)
-        self.set_param_hint('f_s', value=0, min=-1, max=1, vary=False)
-        self.set_param_hint('a_c', value=0, min=0, vary=False)
+        p = self.prefix
+        self.set_param_hint(f'{p}P', value=1, min=1e-15)
+        self.set_param_hint(f'{p}D', value=0.01, min=0, max=0.25)
+        self.set_param_hint(f'{p}W', value=0.1, min=0, max=0.3)
+        self.set_param_hint(f'{p}b', value=0.3, min=0, max=1.0)
+        self.set_param_hint(f'{p}L', value=0.001, min=0, max=1)
+        self.set_param_hint(f'{p}f_c', value=0, min=-1, max=1, vary=False)
+        self.set_param_hint(f'{p}f_s', value=0, min=-1, max=1, vary=False)
+        expr = "{p:s}f_c**2 + {p:s}f_s**2".format(p=self.prefix)
+        self.set_param_hint(f'{p}e',min=0,max=1,expr=expr)
+        self.set_param_hint(f'{p}a_c', value=0, min=0, vary=False)
         expr = "sqrt({prefix:s}D)".format(prefix=self.prefix)
-        self.set_param_hint('k', expr=expr, min=0, max=1)
+        self.set_param_hint(f'{p}k', expr=expr, min=0, max=1)
         expr = "{prefix:s}L/{prefix:s}D".format(prefix=self.prefix)
-        self.set_param_hint('J', expr=expr, min=0)
+        self.set_param_hint(f'{p}J', expr=expr, min=0)
         expr ="sqrt((1+{p:s}k)**2-{p:s}b**2)/{p:s}W/pi".format(p=self.prefix)
-        self.set_param_hint('aR',min=1, expr=expr)
-        self.set_param_hint('rho', min=0, expr = 
-                "0.013418*{p:s}aR**3/{p:s}P**2".format(p=self.prefix) )
-
+        self.set_param_hint(f'{p}aR',min=1, expr=expr)
+        expr ="0.013418*{p:s}aR**3/{p:s}P**2".format(p=self.prefix)
+        self.set_param_hint(f'{p}rho', min=0, expr = expr)
 
 #----------------------
 
 class FactorModel(Model):
     r"""Flux scaling and trend factor model
 
-    f = c*(1 + dfdt*dt + d2fdt2*dt**2 + dfdbg*bg(t)  + dfdcontam*contam(t) +
+    f = c*(1 + dfdt*dt + d2fdt2*dt**2 + dfdbg*bg(t) +
+               dfdcontam*contam(t) + dfdsmear*smear(t) +
+               ramp*deltaT(t)/1e6 +
                dfdx*dx(t) + dfdy*dy(t) +
                d2fdx2*dx(t)**2 + d2f2y2*dy(t)**2 + d2fdxdy*x(t)*dy(t) +
                dfdsinphi*sin(phi(t)) + dfdcosphi*cos(phi(t)) +
                dfdsin2phi*sin(2.phi(t)) + dfdcos2phi*cos(2.phi(t)) + 
                dfdsin3phi*sin(3.phi(t)) + dfdcos3phi*cos(3.phi(t)) ) 
 
-    The detrending coefficients dfdx, etc. are 0 and fixed by default. If
-    any of the coefficients dfdx, d2fdxdy or d2f2x2 is not 0, a function to
+    The detrending coefficients dfdx, etc. are 0 and fixed by default. If any
+    of the coefficients dfdx, d2fdxdy or d2f2x2 is not 0, a function to
     calculate the x-position offset as a function of time, dx(t), must be
     passed as a keyword argument, and similarly for the y-position offset,
     dy(t). For detrending against the spacecraft roll angle, phi(t), the
     functions to be provided as keywords arguments are sinphi(t) and
     cosphi(t). The linear trend dfdbg is proportional to the estimated
     background flux in the aperture, bg(t). The linear trend dfdcontam is
-    proportional to the estimated contamination in the aperture contam(t).
-    The time trend decribed by dfdt and d2fdt2 is calculated using the
-    variable dt = t - median(t).
+    proportional to the estimated contamination in the aperture contam(t). The
+    linear trend dfdsmear is proportional to the estimated smearing correction
+    in the aperture, smear(t). The time trend decribed by dfdt and d2fdt2 is
+    calculated using the variable dt = t - median(t).
 
     """
 
     def __init__(self, independent_vars=['t'], prefix='', nan_policy='raise',
                  dx=None, dy=None, sinphi=None, cosphi=None, bg=None,
-                 contam=None,  **kwargs):
+                 contam=None, smear=None, deltaT=None,  **kwargs):
         kwargs.update({'prefix': prefix, 'nan_policy': nan_policy,
                        'independent_vars': independent_vars})
 
-        def factor(t, c=1.0,dfdt=0, d2fdt2=0, dfdbg=0, dfdcontam=0,
+        def factor(t, c=1.0,dfdt=0, d2fdt2=0, dfdbg=0,
+                dfdcontam=0, dfdsmear=0, ramp=0, 
                 dfdx=0, dfdy=0, d2fdxdy=0, d2fdx2=0, d2fdy2=0,
                 dfdcosphi=0, dfdsinphi=0, dfdcos2phi=0, dfdsin2phi=0,
                 dfdcos3phi=0, dfdsin3phi=0):
@@ -506,6 +521,10 @@ class FactorModel(Model):
                 trend += dfdbg*self.bg(t)
             if dfdcontam != 0:
                 trend += dfdcontam*self.contam(t)
+            if dfdsmear != 0:
+                trend += dfdsmear*self.smear(t)
+            if ramp != 0:
+                trend += ramp*deltaT(t)/1e6
             if dfdx != 0 or d2fdx2 != 0:
                 trend += dfdx*self.dx(t) + d2fdx2*self.dx(t)**2
             if dfdy != 0 or d2fdy2 != 0:
@@ -532,13 +551,14 @@ class FactorModel(Model):
 
         self.bg = bg
         self.contam = contam
+        self.smear = smear
         self.dx = dx
         self.dy = dy
         self.sinphi = sinphi
         self.cosphi = cosphi
         self.set_param_hint('c', min=0)
-        for p in ['dfdt', 'd2fdt2', 'dfdbg', 'dfdcontam',
-                  'dfdx', 'dfdy', 'd2fdx2', 'd2fdxdy',  'd2fdy2', 
+        for p in ['dfdt', 'd2fdt2', 'dfdbg', 'dfdcontam', 'dfdsmear',
+                  'dfdx', 'dfdy', 'd2fdx2', 'd2fdxdy',  'd2fdy2', 'ramp', 
                   'dfdsinphi', 'dfdcosphi', 'dfdcos2phi', 'dfdsin2phi',
                   'dfdcos3phi', 'dfdsin3phi']:
             self.set_param_hint(p, value=0, vary=False)
@@ -548,7 +568,7 @@ class FactorModel(Model):
         pars = self.make_params()
 
         pars['%sc' % self.prefix].set(value=data.median())
-        for p in ['dfdt', 'd2fdt2' 'dfdbg', 'dfdcontam',
+        for p in ['dfdt', 'd2fdt2' 'dfdbg', 'dfdcontam', 'dfdsmear',
                 'dfdx', 'dfdy', 'd2fdx2', 'd2fdy2', 
                 'dfdsinphi', 'dfdcosphi', 'dfdcos2phi', 'dfdsin2phi',
                 'dfdcos3phi', 'dfdsin3phi']:
@@ -596,18 +616,19 @@ class ThermalPhaseModel(Model):
         self._set_paramhints_prefix()
 
     def _set_paramhints_prefix(self):
-        self.set_param_hint('P', min=1e-15)
-        self.set_param_hint('a_th', value=0)
-        self.set_param_hint('b_th', value=0)
-        self.set_param_hint('c_th', value=0, min=0)
-        expr = "hypot({p:s}a_th,{p:s}b_th)".format(p=self.prefix)
-        self.set_param_hint('A', expr=expr)
-        expr = "{p:s}c_th+({p:s}a_th+{p:s}b_th+{p:s}A)/2".format(p=self.prefix)
-        self.set_param_hint('Fmax', expr=expr, min=0)
+        p = self.prefix
+        self.set_param_hint(f'{p}P', min=1e-15)
+        self.set_param_hint(f'{p}a_th', value=0)
+        self.set_param_hint(f'{p}b_th', value=0)
+        self.set_param_hint(f'{p}c_th', value=0, min=0)
+        expr="hypot({p:s}a_th,{p:s}b_th)".format(p=self.prefix)
+        self.set_param_hint(f'{p}A', expr=expr)
+        expr="{p:s}c_th+({p:s}a_th+{p:s}b_th+{p:s}A)/2".format(p=self.prefix)
+        self.set_param_hint(f'{p}Fmax', expr=expr, min=0)
         expr = "{p:s}Fmax - {p:s}A".format(p=self.prefix)
-        self.set_param_hint('Fmin', expr=expr, min=0)
+        self.set_param_hint(f'{p}Fmin', expr=expr, min=0)
         expr = "arctan2({p:s}b_th,-{p:s}a_th)/(2*pi)".format(p=self.prefix)
-        self.set_param_hint('ph_max', expr=expr)
+        self.set_param_hint(f'{p}ph_max', expr=expr)
 
     __init__.__doc__ = COMMON_INIT_DOC
 
@@ -658,12 +679,13 @@ class ReflectionModel(Model):
         self._set_paramhints_prefix()
 
     def _set_paramhints_prefix(self):
-        self.set_param_hint('P', min=1e-15)
-        self.set_param_hint('A_g', value=0.5, min=0, max=1)
-        self.set_param_hint('r_p', min=0, max=1)
-        self.set_param_hint('f_c', value=0, vary=False, min=-1, max=1)
-        self.set_param_hint('f_s', value=0, vary=False, min=-1, max=1)
-        self.set_param_hint('sini', value=1, vary=False, min=0, max=1)
+        p = self.prefix
+        self.set_param_hint(f'{p}P',value=1, min=1e-15)
+        self.set_param_hint(f'{p}A_g', value=0.5, min=0, max=1)
+        self.set_param_hint(f'{p}r_p', min=0, max=1)
+        self.set_param_hint(f'{p}f_c', value=0, vary=False, min=-1, max=1)
+        self.set_param_hint(f'{p}f_s', value=0, vary=False, min=-1, max=1)
+        self.set_param_hint(f'{p}sini', value=1, vary=False, min=0, max=1)
 
     __init__.__doc__ = COMMON_INIT_DOC
 
@@ -699,21 +721,19 @@ class RVModel(Model):
         self._set_paramhints_prefix()
 
     def _set_paramhints_prefix(self):
-        self.set_param_hint('P', min=1e-15)
-        self.set_param_hint('K', min=1e-15)
-        self.set_param_hint('f_c', value=0, vary=False, min=-1, max=1)
-        self.set_param_hint('f_s', value=0, vary=False, min=-1, max=1)
-        self.set_param_hint('sini', value=1, vary=False, min=0, max=1)
+        p = self.prefix
+        self.set_param_hint(f'{p}P', min=1e-15)
+        self.set_param_hint(f'{p}K', min=1e-15)
+        self.set_param_hint(f'{p}f_c', value=0, vary=False, min=-1, max=1)
+        self.set_param_hint(f'{p}f_s', value=0, vary=False, min=-1, max=1)
         expr = "{p:s}f_c**2 + {p:s}f_s**2".format(p=self.prefix)
-        self.set_param_hint('{p:s}e'.format(p=self.prefix), expr=expr, 
-                min=0, max=1)
+        self.set_param_hint(f'{p}e',min=0,max=1,expr=expr)
+        self.set_param_hint(f'{p}sini', value=1, vary=False, min=0, max=1)
         expr = "180*atan2({p:s}f_s, {p:s}f_c)/pi".format(p=self.prefix)
-        self.set_param_hint('{p:s}omega'.format(p=self.prefix), expr=expr) 
+        self.set_param_hint(f'{p}omega', expr=expr) 
 
     __init__.__doc__ = COMMON_INIT_DOC
 
-
-
 #----------------------
 
 class RVCompanion(Model):
@@ -746,16 +766,17 @@ class RVCompanion(Model):
         self._set_paramhints_prefix()
 
     def _set_paramhints_prefix(self):
-        self.set_param_hint('P', min=1e-15)
-        self.set_param_hint('K', min=1e-15)
-        self.set_param_hint('f_c', value=0, vary=False, min=-1, max=1)
-        self.set_param_hint('f_s', value=0, vary=False, min=-1, max=1)
-        self.set_param_hint('sini', value=1, vary=False, min=0, max=1)
+        p = self.prefix
+        self.set_param_hint(f'{p}P', min=1e-15)
+        self.set_param_hint(f'{p}K', min=1e-15)
+        self.set_param_hint(f'{p}f_c', value=0, vary=False, min=-1, max=1)
+        self.set_param_hint(f'{p}f_s', value=0, vary=False, min=-1, max=1)
+        self.set_param_hint(f'{p}sini', value=1, vary=False, min=0, max=1)
         expr = "{p:s}f_c**2 + {p:s}f_s**2".format(p=self.prefix)
-        self.set_param_hint('{p:s}e'.format(p=self.prefix), expr=expr, 
+        self.set_param_hint(f'{p}e'.format(p=self.prefix), expr=expr, 
                 min=0, max=1)
         expr = "180*atan2({p:s}f_s, {p:s}f_c)/pi".format(p=self.prefix)
-        self.set_param_hint('{p:s}omega'.format(p=self.prefix), expr=expr)
+        self.set_param_hint(f'{p}omega'.format(p=self.prefix), expr=expr)
 
     __init__.__doc__ = COMMON_INIT_DOC
 
@@ -858,6 +879,7 @@ class PlanetModel(Model):
             om = np.arctan2(f_s, f_c)*180/np.pi
             # Star-planet apparent separation and mask eclipses/transits
             z,m = t2z(t, T_0, P, sini, r_star, ecc, om, returnMask=True)
+            if False in np.isfinite(z): return np.ones_like(t)
             # Set z values where planet is behind star  1 to a large nominal
             # value for calculation of the transit
             zt = z + 0   # copy 
@@ -876,25 +898,33 @@ class PlanetModel(Model):
         self._set_paramhints_prefix()
 
     def _set_paramhints_prefix(self):
-        self.set_param_hint('P', min=1e-15)
-        self.set_param_hint('D', min=0, max=0.25)
-        self.set_param_hint('W', min=0, max=0.3)
-        self.set_param_hint('b', min=0, max=1.0)
-        self.set_param_hint('F_min', min=0)
-        self.set_param_hint('ph_off', min=-0.5, max=0.5)
-        self.set_param_hint('f_c', value=0, min=-1, max=1, vary=False)
-        self.set_param_hint('f_s', value=0, min=-1, max=1, vary=False)
-        self.set_param_hint('h_1', value=0.7224, min=0, max=1, vary=False)
-        self.set_param_hint('h_2', value=0.6713, min=0, max=1, vary=False)
-        self.set_param_hint('a_c', value=0, min=0, vary=False)
+        p = self.prefix
+        self.set_param_hint(f'{p}P', value=1, min=1e-15)
+        self.set_param_hint(f'{p}D', value=0.01, min=0, max=0.25)
+        self.set_param_hint(f'{p}W', value=0.1, min=0, max=0.3)
+        self.set_param_hint(f'{p}b', value=0.3, min=0, max=1.0)
+        self.set_param_hint(f'{p}F_min', value=0, min=0)
+        self.set_param_hint(f'{p}F_max', value=0, min=0)
+        self.set_param_hint(f'{p}ph_off', min=-0.5, max=0.5)
+        self.set_param_hint(f'{p}f_c', value=0, min=-1, max=1, vary=False)
+        self.set_param_hint(f'{p}f_s', value=0, min=-1, max=1, vary=False)
+        expr = "{p:s}f_c**2 + {p:s}f_s**2".format(p=self.prefix)
+        self.set_param_hint(f'{p}e',min=0,max=1,expr=expr)
+        self.set_param_hint(f'{p}h_1', value=0.7224, min=0, max=1, vary=False)
+        self.set_param_hint(f'{p}h_2', value=0.6713, min=0, max=1, vary=False)
+        expr = "(1-{p:s}h_2)**2".format(p=self.prefix)
+        self.set_param_hint(f'{p}q_1',min=0,max=1,expr=expr)
+        expr = "({p:s}h_1-{p:s}h_2)/(1-{p:s}h_2)".format(p=self.prefix)
+        self.set_param_hint(f'{p}q_2',min=0,max=1,expr=expr)
+        self.set_param_hint(f'{p}a_c', value=0, min=0, vary=False)
         expr = "sqrt({prefix:s}D)".format(prefix=self.prefix)
-        self.set_param_hint('k', expr=expr, min=0, max=1)
+        self.set_param_hint(f'{p}k', expr=expr, min=0, max=1)
         expr ="sqrt((1+{p:s}k)**2-{p:s}b**2)/{p:s}W/pi".format(p=self.prefix)
-        self.set_param_hint('aR',min=1, expr=expr)
+        self.set_param_hint(f'{p}aR',min=1, expr=expr)
         expr = "{prefix:s}F_max-{prefix:s}F_min".format(prefix=self.prefix)
-        self.set_param_hint('A', expr=expr)
+        self.set_param_hint(f'{p}A', expr=expr)
         expr = "0.013418*{p:s}aR**3/{p:s}P**2".format(p=self.prefix)
-        self.set_param_hint('rho', min=0, expr = expr)
+        self.set_param_hint(f'{p}rho', min=0, expr = expr)
 
 #----------------------
 
@@ -912,8 +942,10 @@ class EBLMModel(Model):
     includes the correction a_c for the light travel time across the orbit,
     i.e., for a circular orbit the time of mid-eclipse is (T_0 + 0.5*P) + a_c.
 
-    **N.B.** a_c must have the same units as P. The power-2 law is used to model
-    the limb-darkening of star 1. Limb-darkening on star 2 is ignored.
+    **N.B.** a_c must have the same units as P.
+
+    The power-2 law is used to model the limb-darkening of star 1.
+    Limb-darkening on star 2 is ignored.
 
     The following parameters are defined for convenience:
 
@@ -926,12 +958,12 @@ class EBLMModel(Model):
     :param P:   - orbital period
     :param D:   - (R_2/R_1)**2 = k**2
     :param W:   - (R_1/a)*sqrt((1+k)**2 - b**2)/pi
-    :param b:    - a*cos(i)/R_1
+    :param b:   - a*cos(i)/R_1
     :param L:   - Depth of eclipse
     :param f_c: - sqrt(ecc).cos(omega)
     :param f_s: - sqrt(ecc).sin(omega)
-    :param h_1:  - I(0.5) = 1 - c*(1-0.5**alpha)
-    :param h_2:  - I(0.5) - I(0) = c*0.5**alpha
+    :param h_1: - I(0.5) = 1 - c*(1-0.5**alpha)
+    :param h_2: - I(0.5) - I(0) = c*0.5**alpha
     :param a_c: - correction for light travel time across the orbit
 
     """
@@ -965,11 +997,13 @@ class EBLMModel(Model):
             if ecc > 0.95 : return np.ones_like(t)
             om = np.arctan2(f_s, f_c)*180/np.pi
             z,m = t2z(t, T_0, P, sini, r_star, ecc, om, returnMask=True)
+            if False in np.isfinite(z): return np.ones_like(t)
             # Set z values where star 2 is behind star  1 to a large nominal
             # value for calculation of the transit
             z[m] = 100
             lc =  qpower2(z, k, c2, a2)
             z,m = t2z(t-a_c, T_0, P, sini, r_star, ecc, om, returnMask=True)
+            if False in np.isfinite(z): return np.ones_like(t)
             # Set z values where star  1 is behind star 2 to a large nominal
             # value for calculation of the eclipse
             z[~m]  = 100
@@ -979,23 +1013,31 @@ class EBLMModel(Model):
         self._set_paramhints_prefix()
 
     def _set_paramhints_prefix(self):
-        self.set_param_hint('P', min=1e-15)
-        self.set_param_hint('D', min=0, max=0.25)
-        self.set_param_hint('W', min=0, max=0.3)
-        self.set_param_hint('b', min=0, max=1.0)
-        self.set_param_hint('L', min=0, max=1)
-        self.set_param_hint('f_c', value=0, min=-1, max=1, vary=False)
-        self.set_param_hint('f_s', value=0, min=-1, max=1, vary=False)
-        self.set_param_hint('h_1', value=0.7224, min=0, max=1, vary=False)
-        self.set_param_hint('h_2', value=0.6713, min=0, max=1, vary=False)
-        self.set_param_hint('a_c', value=0, min=0, vary=False)
+        p = self.prefix
+        self.set_param_hint(f'{p}P', value=1, min=1e-15)
+        self.set_param_hint(f'{p}D', value=0.01, min=0, max=0.25)
+        self.set_param_hint(f'{p}W', value=0.1, min=0, max=0.3)
+        self.set_param_hint(f'{p}b', value=0.3, min=0, max=1.0)
+        self.set_param_hint(f'{p}L', value=0.001, min=0, max=1)
+        self.set_param_hint(f'{p}f_c', value=0, min=-1, max=1, vary=False)
+        self.set_param_hint(f'{p}f_s', value=0, min=-1, max=1, vary=False)
+        expr = "{p:s}f_c**2 + {p:s}f_s**2".format(p=self.prefix)
+        self.set_param_hint(f'{p}e',min=0,max=1,expr=expr)
+        self.set_param_hint(f'{p}h_1', value=0.7224,min=0,max=1,vary=False)
+        self.set_param_hint(f'{p}h_2', value=0.6713,min=0,max=1,vary=False)
+        expr = "(1-{p:s}h_2)**2".format(p=self.prefix)
+        self.set_param_hint(f'{p}q_1',min=0,max=1,expr=expr)
+        expr = "({p:s}h_1-{p:s}h_2)/(1-{p:s}h_2)".format(p=self.prefix)
+        self.set_param_hint(f'{p}q_2',min=0,max=1,expr=expr)
+        self.set_param_hint(f'{p}a_c', value=0, min=0, vary=False)
         expr = "sqrt({prefix:s}D)".format(prefix=self.prefix)
-        self.set_param_hint('k', expr=expr, min=0, max=1)
+        self.set_param_hint(f'{p}k', expr=expr, min=0, max=1)
         expr = "{prefix:s}L/{prefix:s}D".format(prefix=self.prefix)
-        self.set_param_hint('J', expr=expr, min=0)
+        self.set_param_hint(f'{p}J', expr=expr, min=0)
         expr ="sqrt((1+{p:s}k)**2-{p:s}b**2)/{p:s}W/pi".format(p=self.prefix)
-        self.set_param_hint('aR',min=1, expr=expr)
-
+        self.set_param_hint(f'{p}aR',min=1, expr=expr)
+        expr ="0.013418*{p:s}aR**3/{p:s}P**2".format(p=self.prefix)
+        self.set_param_hint(f'{p}rho', min=0, expr = expr)
 
 #----------------------
 
